@@ -4,6 +4,7 @@
  * Copyright (C) 2005 John (J5) Palmieri <johnp@redhat.com>
  * Copyright (C) 2010 Red Hat, Inc.
  * Copyright (C) 2011 Perberos <perberos@gmail.com>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +34,7 @@
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
+#ifdef HAVE_X11
 #include <X11/Xproto.h>
 
 #include <X11/Xlib.h>
@@ -42,6 +44,7 @@
 
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
+#endif /* HAVE_X11 */
 
 #include "daemon.h"
 #include "engines.h"
@@ -52,8 +55,6 @@
 #define MAX_NOTIFICATIONS 20
 #define IMAGE_SIZE 48
 #define IDLE_SECONDS 30
-#define NOTIFICATION_BUS_NAME      "org.freedesktop.Notifications"
-#define NOTIFICATION_BUS_PATH      "/org/freedesktop/Notifications"
 
 #define NW_GET_NOTIFY_ID(nw) \
 	(GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(nw), "_notify_id")))
@@ -84,20 +85,24 @@ const PopupNotifyStackLocation popup_stack_locations[] = {
 #define POPUP_STACK_DEFAULT_INDEX 3     /* XXX Hack! */
 
 typedef struct {
-	NotifyDaemon* daemon;
-	GTimeVal expiration;
-	GTimeVal paused_diff;
-	guint id;
-	GtkWindow* nw;
-	Window src_window_xid;
-	guint   has_timeout : 1;
-	guint   paused : 1;
+	NotifyDaemon *daemon;
+	gint64        expiration;
+	GTimeSpan     paused_diff;
+	guint         id;
+	GtkWindow    *nw;
+	guint         has_timeout : 1;
+	guint         paused : 1;
+#ifdef HAVE_X11
+	Window        src_window_xid;
+#endif /* HAVE_X11 */
 } NotifyTimeout;
 
 typedef struct {
 	NotifyStack** stacks;
-	int n_stacks;
+	gsize n_stacks;
+#ifdef HAVE_X11
 	Atom workarea_atom;
+#endif /* HAVE_X11 */
 } NotifyScreen;
 
 struct _NotifyDaemon {
@@ -127,19 +132,23 @@ typedef struct {
 static void notify_daemon_finalize(GObject* object);
 static void _notification_destroyed_cb(GtkWindow* nw, NotifyDaemon* daemon);
 static void _close_notification(NotifyDaemon* daemon, guint id, gboolean hide_notification, NotifydClosedReason reason);
-static GdkFilterReturn _notify_x11_filter(GdkXEvent* xevent, GdkEvent* event, NotifyDaemon* daemon);
 static void _emit_closed_signal(GtkWindow* nw, NotifydClosedReason reason);
 static void _action_invoked_cb(GtkWindow* nw, const char* key);
 static NotifyStackLocation get_stack_location_from_string(const gchar *slocation);
+
+#ifdef HAVE_X11
+static GdkFilterReturn _notify_x11_filter(GdkXEvent* xevent, GdkEvent* event, NotifyDaemon* daemon);
 static void sync_notification_position(NotifyDaemon* daemon, GtkWindow* nw, Window source);
 static void monitor_notification_source_windows(NotifyDaemon* daemon, NotifyTimeout* nt, Window source);
-static GParamSpec *properties[LAST_PROP] = { NULL };
+#endif /* HAVE_X11 */
 
 static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object, GDBusMethodInvocation *invocation, const gchar *app_name, guint id, const gchar *icon, const gchar *summary, const gchar *body, const gchar *const *actions, GVariant *hints, gint timeout, gpointer user_data);
 static gboolean notify_daemon_close_notification_handler(NotifyDaemonNotifications *object, GDBusMethodInvocation *invocation, guint arg_id, gpointer user_data);
 static gboolean notify_daemon_get_capabilities( NotifyDaemonNotifications *object, GDBusMethodInvocation *invocation);
 static gboolean notify_daemon_get_server_information (NotifyDaemonNotifications *object, GDBusMethodInvocation *invocation, gpointer user_data);
 
+static GParamSpec *properties[LAST_PROP] = { NULL };
+
 G_DEFINE_TYPE(NotifyDaemon, notify_daemon, G_TYPE_OBJECT);
 
 static void bus_acquired_handler_cb (GDBusConnection *connection,
@@ -308,27 +317,27 @@ static void on_screen_monitors_changed(GdkScreen* screen, NotifyDaemon* daemon)
 
 	n_monitors = gdk_display_get_n_monitors(display);
 
-	if (n_monitors > nscreen->n_stacks)
+	if (n_monitors > (int) nscreen->n_stacks)
 	{
 		/* grow */
-		nscreen->stacks = g_renew(NotifyStack *, nscreen->stacks, n_monitors);
+		nscreen->stacks = g_renew(NotifyStack *, nscreen->stacks, (gsize) n_monitors);
 
 		/* add more stacks */
-		for (i = nscreen->n_stacks; i < n_monitors; i++)
+		for (i = (int) nscreen->n_stacks; i < n_monitors; i++)
 		{
 			create_stack_for_monitor(daemon, screen, gdk_display_get_monitor (display, i));
 		}
 
-		nscreen->n_stacks = n_monitors;
+		nscreen->n_stacks = (gsize) n_monitors;
 	}
-	else if (n_monitors < nscreen->n_stacks)
+	else if (n_monitors < (int) nscreen->n_stacks)
 	{
 		NotifyStack* last_stack;
 
 		last_stack = nscreen->stacks[n_monitors - 1];
 
 		/* transfer items before removing stacks */
-		for (i = n_monitors; i < nscreen->n_stacks; i++)
+		for (i = n_monitors; i < (int) nscreen->n_stacks; i++)
 		{
 			NotifyStack* stack = nscreen->stacks[i];
 			GList* windows = g_list_copy(notify_stack_get_windows(stack));
@@ -348,8 +357,8 @@ static void on_screen_monitors_changed(GdkScreen* screen, NotifyDaemon* daemon)
 		}
 
 		/* remove the extra stacks */
-		nscreen->stacks = g_renew(NotifyStack*, nscreen->stacks, n_monitors);
-		nscreen->n_stacks = n_monitors;
+		nscreen->stacks = g_renew(NotifyStack*, nscreen->stacks, (gsize) n_monitors);
+		nscreen->n_stacks = (gsize) n_monitors;
 	}
 }
 
@@ -357,12 +366,13 @@ static void create_stacks_for_screen(NotifyDaemon* daemon, GdkScreen *screen)
 {
 	GdkDisplay     *display;
 	NotifyScreen* nscreen;
-	int i;
+	int i, n_monitors;
 
 	nscreen = daemon->screen;
 	display = gdk_screen_get_display (screen);
+	n_monitors = gdk_display_get_n_monitors(display);
 
-	nscreen->n_stacks = gdk_display_get_n_monitors(display);
+	nscreen->n_stacks = (gsize) n_monitors;
 
 	nscreen->stacks = g_renew(NotifyStack*, nscreen->stacks, nscreen->n_stacks);
 
@@ -372,6 +382,7 @@ static void create_stacks_for_screen(NotifyDaemon* daemon, GdkScreen *screen)
 	}
 }
 
+#ifdef HAVE_X11
 static GdkFilterReturn screen_xevent_filter(GdkXEvent* xevent, GdkEvent* event, NotifyScreen* nscreen)
 {
 	XEvent* xev = (XEvent*) xevent;
@@ -388,12 +399,12 @@ static GdkFilterReturn screen_xevent_filter(GdkXEvent* xevent, GdkEvent* event,
 
 	return GDK_FILTER_CONTINUE;
 }
+#endif /* HAVE_X11 */
 
 static void create_screen(NotifyDaemon* daemon)
 {
     GdkDisplay *display;
     GdkScreen  *screen;
-    GdkWindow  *gdkwindow;
 
 	g_assert(daemon->screen == NULL);
 
@@ -404,11 +415,17 @@ static void create_screen(NotifyDaemon* daemon)
 
 	daemon->screen = g_new0(NotifyScreen, 1);
 
-	daemon->screen->workarea_atom = XInternAtom(GDK_DISPLAY_XDISPLAY (display), "_NET_WORKAREA", True);
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+	{
+		GdkWindow  *gdkwindow;
 
-	gdkwindow = gdk_screen_get_root_window(screen);
-	gdk_window_add_filter(gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->screen);
-	gdk_window_set_events(gdkwindow, gdk_window_get_events(gdkwindow) | GDK_PROPERTY_CHANGE_MASK);
+		daemon->screen->workarea_atom = XInternAtom(GDK_DISPLAY_XDISPLAY (display), "_NET_WORKAREA", True);
+		gdkwindow = gdk_screen_get_root_window(screen);
+		gdk_window_add_filter(gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->screen);
+		gdk_window_set_events(gdkwindow, gdk_window_get_events(gdkwindow) | GDK_PROPERTY_CHANGE_MASK);
+	}
+#endif /* HAVE_X11 */
 
 	create_stacks_for_screen(daemon, screen);
 }
@@ -477,7 +494,6 @@ static void destroy_screen(NotifyDaemon* daemon)
 {
 	GdkDisplay *display;
 	GdkScreen  *screen;
-	GdkWindow  *gdkwindow;
 	gint        i;
 
 	display = gdk_display_get_default();
@@ -487,8 +503,15 @@ static void destroy_screen(NotifyDaemon* daemon)
 										  G_CALLBACK (on_screen_monitors_changed),
 										  daemon);
 
-	gdkwindow = gdk_screen_get_root_window (screen);
-	gdk_window_remove_filter (gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->screen);
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+	{
+		GdkWindow  *gdkwindow;
+		gdkwindow = gdk_screen_get_root_window (screen);
+		gdk_window_remove_filter (gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->screen);
+	}
+#endif /* HAVE_X11 */
+
 	for (i = 0; i < daemon->screen->n_stacks; i++) {
 		 g_clear_pointer (&daemon->screen->stacks[i], notify_stack_destroy);
 	}
@@ -506,10 +529,13 @@ static void notify_daemon_finalize(GObject* object)
 
 	daemon = NOTIFY_DAEMON(object);
 
-	if (g_hash_table_size(daemon->monitored_window_hash) > 0)
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
+		g_hash_table_size(daemon->monitored_window_hash) > 0)
 	{
 		gdk_window_remove_filter(NULL, (GdkFilterFunc) _notify_x11_filter, daemon);
 	}
+#endif /* HAVE_X11 */
 
 	if (daemon->skeleton != NULL)
 	{
@@ -628,6 +654,7 @@ static void _notification_destroyed_cb(GtkWindow* nw, NotifyDaemon* daemon)
 	_close_notification(daemon, NW_GET_NOTIFY_ID(nw), FALSE, NOTIFYD_CLOSED_EXPIRED);
 }
 
+#ifdef HAVE_X11
 typedef struct {
 	NotifyDaemon* daemon;
 	gint id;
@@ -731,12 +758,12 @@ static GdkFilterReturn _notify_x11_filter(GdkXEvent* xevent, GdkEvent* event, No
 
 	return GDK_FILTER_CONTINUE;
 }
+#endif /* HAVE_X11 */
 
 static void _mouse_entered_cb(GtkWindow* nw, GdkEventCrossing* event, NotifyDaemon* daemon)
 {
 	NotifyTimeout* nt;
 	guint id;
-	GTimeVal now;
 
 	if (event->detail == GDK_NOTIFY_INFERIOR)
 	{
@@ -747,16 +774,7 @@ static void _mouse_entered_cb(GtkWindow* nw, GdkEventCrossing* event, NotifyDaem
 	nt = (NotifyTimeout*) g_hash_table_lookup(daemon->notification_hash, &id);
 
 	nt->paused = TRUE;
-	g_get_current_time(&now);
-
-	nt->paused_diff.tv_usec = nt->expiration.tv_usec - now.tv_usec;
-	nt->paused_diff.tv_sec = nt->expiration.tv_sec - now.tv_sec;
-
-	if (nt->paused_diff.tv_usec < 0)
-	{
-		nt->paused_diff.tv_usec += G_USEC_PER_SEC;
-		nt->paused_diff.tv_sec--;
-	}
+	nt->paused_diff = nt->expiration - g_get_monotonic_time ();
 }
 
 static void _mouse_exitted_cb(GtkWindow* nw, GdkEventCrossing* event, NotifyDaemon* daemon)
@@ -774,21 +792,18 @@ static void _mouse_exitted_cb(GtkWindow* nw, GdkEventCrossing* event, NotifyDaem
 
 static gboolean _is_expired(gpointer key, NotifyTimeout* nt, gboolean* phas_more_timeouts)
 {
-	time_t now_time;
-	time_t expiration_time;
-	GTimeVal now;
+	gint64     now;
+	GTimeSpan  time_span;
 
 	if (!nt->has_timeout)
 	{
 		return FALSE;
 	}
 
-	g_get_current_time(&now);
+	now = g_get_monotonic_time ();
+	time_span = nt->expiration - now;
 
-	expiration_time = (nt->expiration.tv_sec * 1000) + (nt->expiration.tv_usec / 1000);
-	now_time = (now.tv_sec * 1000) + (now.tv_usec / 1000);
-
-	if (now_time > expiration_time)
+	if (time_span <= 0)
 	{
 		theme_notification_tick(nt->nw, 0);
 		_emit_closed_signal(nt->nw, NOTIFYD_CLOSED_EXPIRED);
@@ -796,18 +811,11 @@ static gboolean _is_expired(gpointer key, NotifyTimeout* nt, gboolean* phas_more
 	}
 	else if (nt->paused)
 	{
-		nt->expiration.tv_usec = nt->paused_diff.tv_usec + now.tv_usec;
-		nt->expiration.tv_sec = nt->paused_diff.tv_sec + now.tv_sec;
-
-		if (nt->expiration.tv_usec >= G_USEC_PER_SEC)
-		{
-			nt->expiration.tv_usec -= G_USEC_PER_SEC;
-			nt->expiration.tv_sec++;
-		}
+		nt->expiration = now + nt->paused_diff;
 	}
 	else
 	{
-		theme_notification_tick(nt->nw, expiration_time - now_time);
+		theme_notification_tick (nt->nw, time_span / G_TIME_SPAN_MILLISECOND);
 	}
 
 	*phas_more_timeouts = TRUE;
@@ -842,6 +850,8 @@ static void _calculate_timeout(NotifyDaemon* daemon, NotifyTimeout* nt, int time
 	}
 	else
 	{
+		gint64 usec;
+
 		nt->has_timeout = TRUE;
 
 		if (timeout == -1)
@@ -851,20 +861,9 @@ static void _calculate_timeout(NotifyDaemon* daemon, NotifyTimeout* nt, int time
 
 		theme_set_notification_timeout(nt->nw, timeout);
 
-		glong usec = timeout * 1000L;  /* convert from msec to usec */
+		usec = (gint64) timeout * G_TIME_SPAN_MILLISECOND;  /* convert from msec to usec */
 
-		/*
-		 * If it's less than 0, wrap around back to MAXLONG.
-		 * g_time_val_add() requires a glong, and anything larger than
-		 * MAXLONG will be treated as a negative value.
-		 */
-		if (usec < 0)
-		{
-			usec = G_MAXLONG;
-		}
-
-		g_get_current_time(&nt->expiration);
-		g_time_val_add(&nt->expiration, usec);
+		nt->expiration = usec + g_get_monotonic_time ();
 
 		if (daemon->timeout_source == 0)
 		{
@@ -911,7 +910,11 @@ static NotifyTimeout* _store_notification(NotifyDaemon* daemon, GtkWindow* nw, i
 
 	_calculate_timeout(daemon, nt, timeout);
 
+#if GLIB_CHECK_VERSION (2, 68, 0)
+	g_hash_table_insert(daemon->notification_hash, g_memdup2(&id, sizeof(guint)), nt);
+#else
 	g_hash_table_insert(daemon->notification_hash, g_memdup(&id, sizeof(guint)), nt);
+#endif
 	remove_exit_timeout(daemon);
 
 	return nt;
@@ -928,6 +931,7 @@ static GdkPixbuf * _notify_daemon_pixbuf_from_data_hint (GVariant *icon_data)
         GVariant       *data_variant;
         gsize           expected_len;
         guchar         *data;
+        gsize           data_size;
         GdkPixbuf      *pixbuf;
 
         g_variant_get (icon_data,
@@ -940,8 +944,7 @@ static GdkPixbuf * _notify_daemon_pixbuf_from_data_hint (GVariant *icon_data)
                        &n_channels,
                        &data_variant);
 
-        expected_len = (height - 1) * rowstride + width
-                * ((n_channels * bits_per_sample + 7) / 8);
+        expected_len = (gsize) ((height - 1) * rowstride + width * ((n_channels * bits_per_sample + 7) / 8));
 
         if (expected_len != g_variant_get_size (data_variant)) {
                 g_warning ("Expected image data to be of length %" G_GSIZE_FORMAT
@@ -951,9 +954,12 @@ static GdkPixbuf * _notify_daemon_pixbuf_from_data_hint (GVariant *icon_data)
                 return NULL;
         }
 
-        data = (guchar *) g_memdup (g_variant_get_data (data_variant),
-                                    g_variant_get_size (data_variant));
-
+        data_size = g_variant_get_size (data_variant);
+#if GLIB_CHECK_VERSION (2, 68, 0)
+        data = (guchar *) g_memdup2 (g_variant_get_data (data_variant), data_size);
+#else
+        data = (guchar *) g_memdup (g_variant_get_data (data_variant), (guint) data_size);
+#endif
         pixbuf = gdk_pixbuf_new_from_data (data,
                                            GDK_COLORSPACE_RGB,
                                            has_alpha,
@@ -1037,12 +1043,12 @@ static GdkPixbuf* _notify_daemon_scale_pixbuf(GdkPixbuf *pixbuf, gboolean no_str
 		int scale_x;
 		int scale_y;
 
-		scale_x = (int) (pw * scale_factor);
-		scale_y = (int) (ph * scale_factor);
+		scale_x = (int) (((float) pw) * scale_factor);
+		scale_y = (int) (((float) ph) * scale_factor);
 		return gdk_pixbuf_scale_simple (pixbuf,
-										scale_x,
-										scale_y,
-										GDK_INTERP_BILINEAR);
+		                                scale_x,
+		                                scale_y,
+		                                GDK_INTERP_BILINEAR);
 	}
 	else
 	{
@@ -1143,13 +1149,16 @@ static gboolean screensaver_active(GtkWidget* nw)
 	return active;
 }
 
+#ifdef HAVE_X11
 static gboolean fullscreen_window_exists(GtkWidget* nw)
 {
 	WnckScreen* wnck_screen;
 	WnckWorkspace* wnck_workspace;
 	GList* l;
 
-		wnck_screen = wnck_screen_get(GDK_SCREEN_XNUMBER(gdk_window_get_screen(gtk_widget_get_window(nw))));
+	g_return_val_if_fail (GDK_IS_X11_DISPLAY (gdk_display_get_default ()), FALSE);
+
+	wnck_screen = wnck_screen_get (GDK_SCREEN_XNUMBER (gdk_window_get_screen (gtk_widget_get_window (nw))));
 
 	wnck_screen_force_update (wnck_screen);
 
@@ -1275,8 +1284,8 @@ static void sync_notification_position(NotifyDaemon* daemon, GtkWindow* nw, Wind
 		return;
 	}
 
-	x += width / 2;
-	y += height / 2;
+	x += (int)width / 2;
+	y += (int)height / 2;
 
 	theme_set_notification_arrow (nw, TRUE, x, y);
 	theme_move_notification (nw, x, y);
@@ -1290,6 +1299,7 @@ static void sync_notification_position(NotifyDaemon* daemon, GtkWindow* nw, Wind
 	 */
 	gtk_widget_queue_draw (GTK_WIDGET (nw));
 }
+#endif /* HAVE_X11 */
 
 GQuark notify_daemon_error_quark(void)
 {
@@ -1314,7 +1324,6 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 	gboolean new_notification = FALSE;
 	gint x = 0;
 	gint y = 0;
-	Window window_xid = None;
 	guint return_id;
 	char* sound_file = NULL;
 	gboolean sound_enabled;
@@ -1322,6 +1331,11 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 	gint i;
 	GdkPixbuf* pixbuf;
 	GSettings* gsettings;
+	gboolean fullscreen_window;
+
+#ifdef HAVE_X11
+	Window window_xid = None;
+#endif /* HAVE_X11 */
 
 	if (g_hash_table_size (daemon->notification_hash) > MAX_NOTIFICATIONS)
 	{
@@ -1382,13 +1396,16 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 	 */
 
 
-	if (g_variant_lookup(hints, "window-xid", "@u", &data))
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
+		g_variant_lookup(hints, "window-xid", "@u", &data))
 	{
 		window_xid = (Window) g_variant_get_uint32 (data);
 		g_variant_unref(data);
-	}
+	} else
+#endif /* HAVE_X11 */
 	/* deal with x, and y hints */
-	else if (g_variant_lookup(hints, "x", "i", &x))
+	if (g_variant_lookup(hints, "x", "i", &x))
 	{
 		if (g_variant_lookup(hints, "y", "i", &y))
 		{
@@ -1487,14 +1504,19 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 			g_object_unref (scaled);
 	}
 
-	if (window_xid != None && !theme_get_always_stack (nw))
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
+	    window_xid != None &&
+	    !theme_get_always_stack (nw))
 	{
 		/*
 		 * Do nothing here if we were passed an XID; we'll call
 		 * sync_notification_position later.
 		 */
 	}
-	else if (use_pos_data && !theme_get_always_stack (nw))
+	else
+#endif /* HAVE_X11 */
+	if (use_pos_data && !theme_get_always_stack (nw))
 	{
 		/*
 		 * Typically, the theme engine will set its own position based on
@@ -1511,21 +1533,27 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 		GdkSeat *seat;
 		GdkDevice *pointer;
 		GdkScreen* screen;
-		gint x, y;
 
 		theme_set_notification_arrow (nw, FALSE, 0, 0);
 
 		/* If the "use-active-monitor" gsettings key is set to TRUE, then
 		 * get the monitor the pointer is at. Otherwise, get the monitor
 		 * number the user has set in gsettings. */
-		if (g_settings_get_boolean(daemon->gsettings, GSETTINGS_KEY_USE_ACTIVE))
+		if (g_settings_get_boolean(daemon->gsettings, GSETTINGS_KEY_USE_ACTIVE_MONITOR))
 		{
+			gint coordinate_x, coordinate_y;
+
 			display = gdk_display_get_default ();
 			seat = gdk_display_get_default_seat (display);
 			pointer = gdk_seat_get_pointer (seat);
 
-			gdk_device_get_position (pointer, &screen, &x, &y);
-			monitor_id = gdk_display_get_monitor_at_point (gdk_screen_get_display (screen), x, y);
+			gdk_device_get_position (pointer,
+			                         &screen,
+			                         &coordinate_x,
+			                         &coordinate_y);
+			monitor_id = gdk_display_get_monitor_at_point (gdk_screen_get_display (screen),
+			                                               coordinate_x,
+			                                               coordinate_y);
 		}
 		else
 		{
@@ -1538,7 +1566,7 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 		{
 			/* screw it - dump it on the last one we'll get
 			 a monitors-changed signal soon enough*/
-			monitor_id = gdk_display_get_monitor (gdk_display_get_default(), daemon->screen->n_stacks - 1);
+			monitor_id = gdk_display_get_monitor (gdk_display_get_default(), (int) daemon->screen->n_stacks - 1);
 		}
 
 		notify_stack_add_window (daemon->screen->stacks[_gtk_get_monitor_num (monitor_id)], nw, new_notification);
@@ -1554,21 +1582,32 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 		return_id = id;
 	}
 
+#ifdef HAVE_X11
 	/*
 	 * If we have a source Window XID, start monitoring the tree
 	 * for changes, and reposition the window based on the source
 	 * window.  We need to do this after return_id is calculated.
 	 */
-	if (window_xid != None && !theme_get_always_stack (nw))
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()) &&
+		window_xid != None &&
+		!theme_get_always_stack (nw))
 	{
 		monitor_notification_source_windows (daemon, nt, window_xid);
 		sync_notification_position (daemon, nw, window_xid);
 	}
+#endif /* HAVE_X11 */
+
+	fullscreen_window = FALSE;
+#ifdef HAVE_X11
+	if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+		fullscreen_window = fullscreen_window_exists (GTK_WIDGET (nw));
+#endif /* HAVE_X11 */
+	/* fullscreen_window is assumed to be false on Wayland, as there is no trivial way to check */
 
 	/* If there is no timeout, show the notification also if screensaver
 	 * is active or there are fullscreen windows
 	 */
-	if (!nt->has_timeout || (!screensaver_active (GTK_WIDGET (nw)) && !fullscreen_window_exists (GTK_WIDGET (nw))))
+	if (!nt->has_timeout || (!screensaver_active (GTK_WIDGET (nw)) && !fullscreen_window))
 	{
 		theme_show_notification (nw);
 
@@ -1579,14 +1618,14 @@ static gboolean notify_daemon_notify_handler(NotifyDaemonNotifications *object,
 	}
 	else
 	{
-		_NotifyPendingClose* data;
+		_NotifyPendingClose *notification_data;
 
 		/* The notification was not shown, so queue up a close
 		 * for it */
-		data = g_new0 (_NotifyPendingClose, 1);
-		data->id = id;
-		data->daemon = g_object_ref (daemon);
-		g_idle_add ((GSourceFunc) _close_notification_not_shown, data);
+		notification_data = g_new0 (_NotifyPendingClose, 1);
+		notification_data->id = id;
+		notification_data->daemon = g_object_ref (daemon);
+		g_idle_add ((GSourceFunc) _close_notification_not_shown, notification_data);
 	}
 
 	g_free (sound_file);
