@@ -4,6 +4,7 @@
  * Copyright (C) 2005 John (J5) Palmieri <johnp@redhat.com>
  * Copyright (C) 2010 Red Hat, Inc.
  * Copyright (C) 2011 Perberos <perberos@gmail.com>
+ * Copyright (C) 2012-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,13 +29,7 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
-#define GSETTINGS_SCHEMA             "org.mate.NotificationDaemon"
-#define GSETTINGS_KEY_THEME          "theme"
-#define GSETTINGS_KEY_POPUP_LOCATION "popup-location"
-#define GSETTINGS_KEY_SOUND_ENABLED  "sound-enabled"
-#define GSETTINGS_KEY_DO_NOT_DISTURB "do-not-disturb"
-#define GSETTINGS_KEY_MONITOR_NUMBER "monitor-number"
-#define GSETTINGS_KEY_USE_ACTIVE     "use-active-monitor"
+#include "constants.h"
 
 #define NOTIFY_TYPE_DAEMON (notify_daemon_get_type())
 G_DECLARE_FINAL_TYPE (NotifyDaemon, notify_daemon, NOTIFY, DAEMON, GObject)
