@@ -3,6 +3,7 @@
  * mnd-daemon.c: This file is part of mate-notification-daemon
  *
  * Copyright (C) 2018 Wu Xiaotian <yetist@gmail.com>
+ * Copyright (C) 2018-2021 MATE Developers
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +79,14 @@ int main (int argc, char *argv[])
 {
 	NotifyDaemon *daemon;
 
+	#if defined(HAVE_X11) && defined(HAVE_WAYLAND)
+		gdk_set_allowed_backends ("wayland,x11");
+	#elif defined(HAVE_WAYLAND)
+		gdk_set_allowed_backends ("wayland");
+	#else
+		gdk_set_allowed_backends ("x11");
+	#endif
+
 	gtk_init(&argc, &argv);
 
 	if (!parse_arguments (&argc, &argv))
